#import <Cocoa/Cocoa.h>
#import <WebKit/WebKit.h>
#import "Controller.h"
#import "ActiveDeveloperLib.h"

@implementation Controller

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification;
 {
    // Create a shared WebHistory object
    WebHistory *myHistory = [[WebHistory alloc] init];
    [WebHistory setOptionalSharedHistory:myHistory];

    // Observe WebHistory notifications
    NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
    [nc addObserver:self selector:@selector(historyDidLoad:) 
               name:WebHistoryLoadedNotification object:myHistory];
    [nc addObserver:self selector:@selector(historyDidRemoveAllItems:) 
               name:WebHistoryAllItemsRemovedNotification object:myHistory];
    [nc addObserver:self selector:@selector(historyDidAddItems:) 
               name:WebHistoryItemsAddedNotification object:myHistory];
    [nc addObserver:self selector:@selector(historyDidRemoveItems:) 
               name:WebHistoryItemsRemovedNotification object:myHistory];
               
	if (ActiveDeveloper_connectToIDE())
		NSLog(@" Application entered an ActiveDeveloper session ... ");
	else	NSLog(@" Application running StandAlone from ActiveDeveloper ...");
 
 }

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender;
 {
	ActiveDeveloper_applicationShouldTerminate(sender);
	return YES;
 }

- (void)dealloc;
 {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [super dealloc];
 }

//
// WebHistory Notification Messages
//
- (void)historyDidLoad:(NSNotification *)aNotification;
 {
    NSLog(@"historyDidLoad: where notification=%@", aNotification);

    // Delete the old menu items
    [self removeAllHistoryMenuItems];

    // Get the new history items
    NSArray *items = [[WebHistory optionalSharedHistory] orderedItemsLastVisitedOnDay:[NSCalendarDate calendarDate]];
    NSEnumerator *enumerator = [items objectEnumerator];
    id historyItem;

    // For each item, create a menu item with the history item as the represented object
    while (historyItem = [enumerator nextObject])
	[self addNewMenuItemForHistoryItem:historyItem];
 }

- (void)historyDidAddItems:(NSNotification *)aNotification;
 {
    NSLog(@"historyDidAddItems: where notification=%@", aNotification);

    // Get the new history items
    NSArray *items = [[aNotification userInfo] objectForKey:WebHistoryItemsKey];
    NSEnumerator *enumerator = [items objectEnumerator];
    id historyItem;

    // For each item, create a menu item with the history item as the represented object
    while (historyItem = [enumerator nextObject])
	[self addNewMenuItemForHistoryItem:historyItem];
 }

- (void)historyDidRemoveItems:(NSNotification *)aNotification;
 {
    NSLog(@"historyDidRemoveItems: where notification=%@", aNotification);

    // Get the removed history items
    NSArray *items = [[aNotification userInfo] objectForKey:WebHistoryItemsKey];
    NSEnumerator *enumerator = [items objectEnumerator];
    id historyItem;

    // For each item, remove the corresponding menu item
    while (historyItem = [enumerator nextObject]){
	[self removeMenuItemForHistoryItem:historyItem];
    }
 }

- (void)historyDidRemoveAllItems:(NSNotification *)aNotification;
 {
    NSLog(@"historyDidRemoveAllItems: where notification=%@", aNotification);
    [self removeAllHistoryMenuItems];
 }


//
// Methods in support of adding and removing menu items
//
- (void)removeAllHistoryMenuItems;
 {		// This method leaves the first menu item which is the Clear action
    int count;
    while ((count = [historyMenu numberOfItems]) != 1)
	[historyMenu removeItemAtIndex:count-1];
 }

- (void)addNewMenuItemForHistoryItem:(WebHistoryItem *)historyItem;
 {
    //NSLog(@"addNewMenuItemForHistoryItem: where title=%@ originalURL=%@ URL=%@", [historyItem title], [historyItem originalURLString], [historyItem URLString]);

    // Create a new menu item, and set its  action and target to invoke goToHistoryItem:
    NSMenuItem *menuItem = [[NSMenuItem alloc] initWithTitle:[historyItem URLString] action:@selector(goToHistoryItem:) keyEquivalent:@""];
    [menuItem setTarget:self];
    [menuItem setRepresentedObject:historyItem];

    // Add it to the menu
    [historyMenu addItem:menuItem];
 }

- (void)removeMenuItemForHistoryItem:(WebHistoryItem *)historyItem;
 {
    // Get the menu item corresponsing to historyItem
    // *** Should really check for the represented object, not the title because itemWithTitle: will return the first match
    // *** unless history items are guarantted not to have the same title?
    NSMenuItem *menuItem = [historyMenu itemWithTitle:[historyItem originalURLString]];
    if ([menuItem representedObject] == historyItem)
	[historyMenu removeItem:menuItem];
    else
	NSLog(@"Opps, multiple menu items with the same title, can't remove the right one!");
 }


//
// History Action Methods
//
- (IBAction)clearHistory:(id)sender;
 {
    [[WebHistory optionalSharedHistory] removeAllItems];
 }


- (IBAction)goToHistoryItem:(id)sender;
 {	// This method will be invoked by the menu item whose represented object is the selected history item
    NSLog(@"goToHistoryItem: where sender=%@", sender);

    // Ask the key window's delegate to load the history item
    NSWindow *keyWindow = [[NSApplication sharedApplication] keyWindow];
    if (keyWindow != nil)
	[[keyWindow delegate] goToHistoryItem:[sender representedObject]];

    // If there is no key window, create a new window to load the history item
    else{
	id aDocument = [[NSDocumentController sharedDocumentController] openUntitledDocumentOfType:@"DocumentType" display:YES];
	NSString *urlString = [[sender representedObject] URLString];
	
	[[[aDocument webView] mainFrame] loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:urlString]]];
    }
 }

@end
